%using Evaluant.PS4VS;
%using Evaluant.PS4VS.Parser;
%using Evaluant.PS4VS.Language;
%using System.Linq;

%namespace Evaluant.PS4VS.Lexer


%x COMMENT
%x EXPANDABLESTRING
%x EXPANDABLESTRINGPARAMETER
%x CMDLETPARAMETERS

%{
		Tokens ComparisonOperator(string txt)
		{
            if (txt[0] == '-') return ComparisonOperator(txt.Substring(1));
			switch(txt[0])
			{
				case 'a':
					if(txt=="as") return Tokens.AS;
					if(txt=="and") return Tokens.AND;
					break;
				case 'b':
					switch(ComparisonOperator(txt.Substring(1)))
					{
						case Tokens.AND:
							return Tokens.BAND;
						case Tokens.OR:
							return Tokens.BOR;
					}
					break;
				case 'c':
					if(txt=="contains") return Tokens.CONTAINS;
					if(txt=="casesensitive") return Tokens.CASESENSITIVE;
					switch(ComparisonOperator(txt.Substring(1)))
					{
						case Tokens.EQ:
							return Tokens.CEQ;
						case Tokens.NE:
							return Tokens.CNE;
						case Tokens.GE:
							return Tokens.CGE;
						case Tokens.GT:
							return Tokens.CGT;
						case Tokens.LT:
							return Tokens.CLT;
						case Tokens.LE:
							return Tokens.CLE;
						case Tokens.LIKE:
							return Tokens.CLIKE;
						case Tokens.NOTLIKE:
							return Tokens.CNOTLIKE;
						case Tokens.MATCH:
							return Tokens.CMATCH;
						case Tokens.NOTMATCH:
							return Tokens.CNOTMATCH;
						case Tokens.CONTAINS:
							return Tokens.CCONTAINS;
						case Tokens.NOTCONTAINS:
							return Tokens.CNOTCONTAINS;
						case Tokens.REPLACE:
							return Tokens.CREPLACE;
					}
					break;
				case 'e':
					if(txt=="eq") return Tokens.EQ;
					break;
				case 'f':
					if(txt=="f") return Tokens.FORMATOPERATOR;
					if(txt=="file") return Tokens.FILE;
					break;
				case 'g':
					if(txt=="ge") return Tokens.GE;
					if(txt=="gt") return Tokens.GT;
					break;
				case 'i':
					if(txt=="isnot") return Tokens.ISNOT;
					if(txt=="is") return Tokens.IS;					
					switch(ComparisonOperator(txt.Substring(1)))
					{
						case Tokens.EQ:
							return Tokens.IEQ;
						case Tokens.NE:
							return Tokens.INE;
						case Tokens.GE:
							return Tokens.IGE;
						case Tokens.GT:
							return Tokens.IGT;
						case Tokens.LT:
							return Tokens.ILT;
						case Tokens.LE:
							return Tokens.ILE;
						case Tokens.LIKE:
							return Tokens.ILIKE;
						case Tokens.NOTLIKE:
							return Tokens.INOTLIKE;
						case Tokens.MATCH:
							return Tokens.IMATCH;
						case Tokens.NOTMATCH:
							return Tokens.INOTMATCH;
						case Tokens.CONTAINS:
							return Tokens.ICONTAINS;
						case Tokens.NOTCONTAINS:
							return Tokens.INOTCONTAINS;
						case Tokens.REPLACE:
							return Tokens.IREPLACE;
					}
					break;
				case 'l':
					if(txt=="le") return Tokens.LE;
					if(txt=="lt") return Tokens.LT;
					if(txt=="like") return Tokens.LIKE;
					break;
				case 'n':
					if(txt=="ne") return Tokens.NE;
					if(txt=="notlike") return Tokens.NOTLIKE;
					if(txt=="notmatch") return Tokens.NOTMATCH;
					if(txt=="notcontains") return Tokens.NOTCONTAINS;
					break;
				case 'o':
					if(txt=="or") return Tokens.OR;
					break;
				case 'r':
					if(txt=="replace") return Tokens.REPLACE;
					if(txt=="regex") return Tokens.REGEX;
					break;
				case 'w':
					if(txt=="wildcard") return Tokens.WILDCARD;
					break;
			}
			return Tokens.LEX_ERROR;
		}

         Tokens GetIdToken(string txt)
         {
            switch (txt[0])
            {
                case 'b':
                    if (txt.Equals("break")) return Tokens.KWBREAK;
                    break;
                case 'c':
                    if (txt.Equals("continue")) return Tokens.KWCONTINUE;
                    break;
                case 'd':
                    if (txt.Equals("do")) return Tokens.KWDO;
                    if (txt.Equals("default")) return Tokens.KWDEFAULT;
					break;
                case 'e':
                    if (txt.Equals("else")) return Tokens.KWELSE;
                    else if (txt.Equals("elseif")) return Tokens.KWELSEIF;
                    break;
                case 'f':
                    if (txt.Equals("for")) return Tokens.KWFOR;
                    if (txt.Equals("foreach")) return Tokens.KWFOREACH;
                    break;
                case 'i':
                    if (txt.Equals("if")) return Tokens.KWIF;
                    break;
                 case 'r':
                    if (txt.Equals("return")) return Tokens.KWRETURN;
                    break;
                case 's':
					if(txt=="switch") return Tokens.KWSWITCH;
                    break;
                case 'v':
                    break;
                case 'w':
                    if (txt.Equals("while")) return Tokens.KWWHILE;
                    break;
                default: 
                    break;
            }
            if(Resolver.GetCommands().Any(command => string.Equals(command.Name,txt,StringComparison.InvariantCultureIgnoreCase)))
				return Tokens.IDENTIFIER;
			return Tokens.UIDENTIFIER;
       }
       
       public int GetToken(string txt, Tokens expectedToken)
       {
			if(!string.IsNullOrEmpty(txt))
				return (int)expectedToken;
			return (int)Tokens.LEX_WHITE;
       }
       
       internal void LoadYylval()
       {
           yylval.str = tokTxt;
           yylloc = new LexLocation(tokLin, tokCol, tokLin, tokECol);
       }
       
       public override void yyerror(string s, params object[] a)
       {
           if (handler != null) handler.AddError(s, tokLin, tokCol, tokLin, tokECol);
       }
%}


White0          [ \t\r\f\v]
White           {White0}|\n
Identifier		[A-Za-z][^ \t\n\r("'@:]*

CmntStart    \/\*
CmntEnd      \*\/
ABStar       [^\*\n]*

%%

[0-9]+										{ return GetToken(yytext,Tokens.NUMBER); }

\#(\ |{ABStar})*							{ BEGIN(COMMENT); return GetToken(yytext,Tokens.LEX_COMMENT); }
<COMMENT>\n+								{ BEGIN(INITIAL); return GetToken(yytext,Tokens.LEX_WHITE); }

{Identifier}								{ Tokens tok=GetIdToken(yytext);if(tok==Tokens.IDENTIFIER || tok==Tokens.UIDENTIFIER)BEGIN(CMDLETPARAMETERS);return (int)tok; }
\-{Identifier}								{ return (int)ComparisonOperator(yytext); }


\@\(										{ return GetToken(yytext,Tokens.ARRAYOPERATOR); }
\@\{										{ return GetToken(yytext,Tokens.HASHOPERATOR); }
\;											{ return (int)';'; }
\,											{ return (int)','; }
\(											{ return (int)'('; }
\)											{ return (int)')'; }
\{											{ return (int)'{'; }
\}											{ return (int)'}'; }
\^											{ return (int)'^'; }
=											{ return (int)'='; }
\+=											{ return GetToken(yytext,Tokens.SELFPLUS); }
\-=											{ return GetToken(yytext,Tokens.SELFMINUS); }
\*=											{ return GetToken(yytext,Tokens.SELFMULT); }
\/=											{ return GetToken(yytext,Tokens.SELFDIV); }
\%=											{ return GetToken(yytext,Tokens.SELFMOD); }


2\>\&1										{ return GetToken(yytext,Tokens.ERRORTOSTDOUT); }
\>\>										{ return GetToken(yytext,Tokens.STDOUTTOAPPENDFILE); }
\>											{ return GetToken(yytext,Tokens.STDOUTTOFILE); }
\<\<										{ return GetToken(yytext,Tokens.APPENDFILETOSTDIN); }
\<											{ return GetToken(yytext,Tokens.FILETOSTDIN); }
\>\|										{ return GetToken(yytext,Tokens.ERRORTOFILE); }
2\>											{ return GetToken(yytext,Tokens.REDIRECTIONOPERATOR); }
2\>\>										{ return GetToken(yytext,Tokens.ERRORTOAPPENDFILE); }
1\>\>										{ return GetToken(yytext,Tokens.STDOUTTOAPPENDFILE); }

(function|filter)							{ return GetToken(yytext,Tokens.FUNCTIONDECLARATION); }
\"											{ BEGIN(EXPANDABLESTRING);return GetToken(yytext,Tokens.EXPANDABLESTRING); }
<EXPANDABLESTRING>\$\w						{ return GetToken(yytext,Tokens.EXPANDABLESTRINGVARIABLE); }
<EXPANDABLESTRING>[^$"]*					{ return GetToken(yytext,Tokens.EXPANDABLESTRINGCONTENT); }
<EXPANDABLESTRING>\"						{ BEGIN(INITIAL); return GetToken(yytext,Tokens.EXPANDABLESTRING); }
\'([^']|\\')*\'								{ return GetToken(yytext,Tokens.LITERALSTRING); }

\${Identifier}								{ return GetToken(yytext,Tokens.VARIABLE); }
\$\{{Identifier}\}							{ return GetToken(yytext,Tokens.VARIABLE); }
\|											{ return GetToken(yytext,Tokens.PIPE); }
\-\-										{ return GetToken(yytext,Tokens.MINUSMINUS); }
\+\+										{ return GetToken(yytext,Tokens.PLUSPLUS); }
\.\.										{ return GetToken(yytext,Tokens.RANGEOPERATOR); }
\.|\:\:|\[									{ return GetToken(yytext,Tokens.LEX_ERROR); }
\+											{ return (int)'+'; }
\-											{ return (int)'-'; }
\[[^\]]+\]									{ return GetToken(yytext,Tokens.ATTRIBUTESPECIFICATION); }
\*											{ return (int)'*'; }
\/											{ return (int)'/'; }
\%											{ return (int)'%'; }
\!											{ return (int)'!'; }
\&\&										{ return GetToken(yytext,Tokens.AMPAMP); }
\|\|										{ return GetToken(yytext,Tokens.BARBAR); }




param										{ return GetToken(yytext,Tokens.PARAMETERTOKEN); }


\.											{ return (int)'.';    }

<CMDLETPARAMETERS>\-{Identifier}			{ return GetToken(yytext,Tokens.PARAMETER); }
<CMDLETPARAMETERS>{Identifier}				{ return GetToken(yytext,Tokens.PARAMETERARGUMENT); }
<CMDLETPARAMETERS>\${Identifier}			{ return (int)Tokens.VARIABLE; }
<CMDLETPARAMETERS>\$\{{Identifier}\}		{ return GetToken(yytext,Tokens.VARIABLE); }
<CMDLETPARAMETERS>\n						{ BEGIN(INITIAL); return GetToken(yytext,Tokens.EOL); }
<CMDLETPARAMETERS>\"						{ BEGIN(EXPANDABLESTRINGPARAMETER);return GetToken(yytext,Tokens.EXPANDABLESTRING); }
<EXPANDABLESTRINGPARAMETER>\$\w				{ return GetToken(yytext,Tokens.EXPANDABLESTRINGVARIABLE); }
<EXPANDABLESTRINGPARAMETER>[^$"]*			{ return GetToken(yytext,Tokens.EXPANDABLESTRINGCONTENT); }
<EXPANDABLESTRINGPARAMETER>\"				{ BEGIN(CMDLETPARAMETERS); return GetToken(yytext,Tokens.EXPANDABLESTRING); }

[A-Za-z][A-Za-z0-9]*\:						{ return GetToken(yytext,Tokens.LOOPLABEL); }


{White0}+                  { return (int)Tokens.LEX_WHITE; }
\n                         { return (int)Tokens.EOL; }
.                          { yyerror("illegal char");
                             return (int)Tokens.LEX_ERROR; }
                             
%{
                      LoadYylval();
%}

%%

/* .... */
