%using Microsoft.VisualStudio.TextManager.Interop
%using Evaluant.PS4VS.Language.Expressions
%namespace Evaluant.PS4VS.Parser
%valuetype ParsedExpression
%partial

/* %expect 5 */


%union {
    public string str;
    public Language.Expressions.IAAST expr;
    public int op;
}

%token<str> IDENTIFIER NUMBER UIDENTIFIER
%token<str> KWIF KWELSE KWELSEIF KWWHILE KWFOR KWCONTINUE KWBREAK KWRETURN KWSWITCH
%token<str> KWFOREACH KWIN LOOPLABEL KWDO KWUNTIL 
%token KWTRAP KWFINALLY
%token REGEX WILDCARD EXACT CASESENSITIVE KWDEFAULT
%token FILE PROPERTYNAME FUNCTIONDECLARATION
%token KWBEGIN KWPROCESS KWEND
%token PARAMETERDECLARATION
%token COMMA

%token AND OR
%token BAND BOR

%token PIPE
%token ATTRIBUTESPECIFICATION
%token<str> PARAMETER PARAMETERARGUMENT
%token<str> VARIABLE

 //%token ',' ';' '(' ')' '{' '}' '=' 
 //%token '+' '-' '*' '/' '!' '&' '|' '^'

%token<str> SELFPLUS SELFMINUS SELFMULT SELFDIV SELFMOD PLUSPLUS MINUSMINUS
%token ERRORTOSTDOUT STDOUTTOAPPENDFILE STDOUTTOFILE APPENDFILETOSTDIN FILETOSTDIN ERRORTOFILE REDIRECTIONOPERATOR ERRORTOAPPENDFILE
%token FORMATOPERATOR RANGEOPERATOR ARRAYOPERATOR HASHOPERATOR
%token UNARYOPERATOR
%token LITERALSTRING EXPANDABLESTRING EXPANDABLESTRINGCONTENT EXPANDABLESTRINGVARIABLE

%token BARBAR AMPAMP EOL
%token<str> EQ NE GT GE LT LE LIKE NOTLIKE MATCH NOTMATCH CONTAINS NOTCONTAINS REPLACE ISNOT IS AS
%token IEQ INE IGT IGE ILT ILE ILIKE INOTLIKE IMATCH INOTMATCH ICONTAINS INOTCONTAINS IREPLACE
%token CEQ CNE CGT CGE CLT CLE CLIKE CNOTLIKE CMATCH CNOTMATCH CCONTAINS CNOTCONTAINS CREPLACE
%token maxParseToken 
%token LEX_WHITE LEX_COMMENT LEX_ERROR

%type<expr> StatementRule CmdletCall 
%type<expr> PipelineRule PipelineRuleOpt
%type<expr> IfStatementRule SwitchStatementRule
%type<expr> ForeachStatementRule ForWhileStatementRule DoWhileStatementRule
%type<expr> FunctionDeclarationRule
%type<expr> ParameterDeclarationRule
%type<expr> FlowControlStatementRule
%type<expr> TrapStatementRule FinallyStatementRule
%type<expr> AssignmentStatementOpt 
%type<expr> SimpleLValue AssignmentStatementRule
%type<expr> ExpressionRule FirstPipelineElement


%type<op> AssignmentOperator

%left '*' '/'
%left '+' '-'

%%

Program
    : StatementListRule
    ;

StatementRule
	: IfStatementRule
	| SwitchStatementRule
	| ForeachStatementRule
	| ForWhileStatementRule
	| DoWhileStatementRule
	| FunctionDeclarationRule
	| ParameterDeclarationRule
	| FlowControlStatementRule
	| TrapStatementRule
	| FinallyStatementRule
	| PipelineRule												
	;
	
StatementBlockRule
	: '{' StatementListRule '}'									{ Match(@1,@3); }
	;
	
StatementListRule
	: StatementRule
	| StatementListRule StatementSeparator StatementRule
	;
	
PipelineRuleOpt
	: /* empty */
	| PipelineRule
	;
	
PipelineRule
	: FirstPipelineElement OtherPipelineElementOpt
	;
	
	
FirstPipelineElement
	: ExpressionRule AssignmentStatementOpt	{ if($2!=null) {((BinaryExpression)$2).Left=$1; $$=$2;} else $$=$1; }
	| CmdletCall
	;
	
AssignmentStatementOpt
	: /* empty */
	| AssignmentOperator PipelineRule		{ $$ = new BinaryExpression(null, (BinaryExpressionType)$1, $2); }
	;
	
OtherPipelineElementOpt
	: /* empty */
	| OtherPipelineElement 
	;
	
OtherPipelineElement
	: PIPE CmdletCall 
	| OtherPipelineElement PIPE CmdletCall
	;
	
AssignmentStatementRule
	: LValueExpression AssignmentOperator PipelineRule
	;
	
LValueExpression
	: LValue
	| LValueExpression LValue
	;
	
LValue
	: SimpleLValue PropertyOrArrayReferenceOperatorOpt
	;
	
PropertyOrArrayReferenceOperatorOpt
	: /* empty */
	| PropertyOrArrayReferenceOperator
	;
	
SimpleLValue
	: AttributeSpecificationsOpt VARIABLE	{ $$ = RegisterVariable($2); }
	;
	
AttributeSpecificationsOpt
	: /* empty */
	| AttributeSpecifications
	;
	
AttributeSpecifications
	: ATTRIBUTESPECIFICATION
	| AttributeSpecifications ATTRIBUTESPECIFICATION
	;
	
CmdletCall
	: CmdletCallProcessIndicator CmdletName CmdletParametersOpt		{ $$=new CmdletCall($2.str,$3.expr); }
	;
	
CmdletCallProcessIndicator
	: /* empty */
	| '&'
	| '.'
	;
	
CmdletName
	: IDENTIFIER			{ Sink.StartName(TextSpan(@1),$1); Sink.StartParameters(TextSpan(@1)); }
	| UIDENTIFIER			
	| ExpressionRule
	;
	
CmdletParametersOpt
	: /* empty */
	| CmdletParameters
	;
	
CmdletParameters
	: CmdletParameter
	| CmdletParameters { Sink.NextParameter(TextSpan(@1)); } CmdletParameter
	;
	
CmdletParameter
	: PARAMETER
	| PARAMETERARGUMENT				
	| PostfixOperatorRule
	| RedirectionRule
	;
	
StatementSeparatorOpt
	: /* empty */
	| StatementSeparator
	;
	
StatementSeparator
	: ';'
	| BARBAR
	| AMPAMP
	| EOL
	| StatementSeparator EOL
	; 

RedirectionRule
	: RedirectionOperator PropertyOrArrayReferenceRule
	;
	
IfStatementRule
	: KWIF '(' PipelineRule ')' StatementSeparatorOpt StatementBlockRule ElseIfsOpt ElseOpt { Match(@2,@4); }
	;
	
ElseIfsOpt
	: /* empty */
	| ElseIfs
	;
	
ElseIfs
	: ElseIf StatementSeparatorOpt
	| ElseIfs ElseIf StatementSeparatorOpt
	;

ElseIf
	: KWELSEIF '(' PipelineRule ')' StatementBlockRule StatementSeparatorOpt { Match(@2,@4); }
	;
	
ElseOpt
	: /* empty */
	| KWELSE StatementBlockRule StatementSeparatorOpt
	;
	
SwitchStatementRule
	: KWSWITCH SwitchMatchTypeOpt CaseSensitiveOpt SwitchTest '{' SwitchCases '}' { Match(@5,@7); }
	;
	
SwitchMatchTypeOpt
	: /* empty */
	| REGEX
	| WILDCARD
	| EXACT
	;

CaseSensitiveOpt
	: /* empty */
	| CASESENSITIVE
	;
	
SwitchTest
	: FILE PropertyOrArrayReferenceRule 
	| '(' PipelineRule ')'						{ Match(@1,@3); }
	;

SwitchCases
	: SwitchCase
	| SwitchCases SwitchCase
	;

SwitchCase
	: KWDEFAULT StatementBlockRule
	| PARAMETERARGUMENT StatementBlockRule
	| PropertyOrArrayReferenceRule StatementBlockRule
	| StatementBlockRule StatementBlockRule
	;

ForeachStatementRule
	: LoopLabelOpt KWFOREACH '(' VARIABLE { RegisterVariable($4); } KWIN PipelineRule ')' StatementBlockRule { Match(@3,@6); }
	;
	
LoopLabelOpt
	: /* empty */
	| LOOPLABEL
	;
	
ForWhileStatementRule
	: LoopLabelOpt KWWHILE '(' PipelineRule ')' StatementBlockRule { Match(@3,@5); }
	| LoopLabelOpt KWFOR '(' PipelineRuleOpt ';' PipelineRuleOpt ';' PipelineRuleOpt ')' StatementBlockRule
	;
	
DoWhileStatementRule
	: LoopLabelOpt KWDO StatementBlockRule DoWhileKeyword '(' PipelineRule ')' { Match(@5,@7); }
	;
	
DoWhileKeyword
	: KWWHILE
	| KWUNTIL
	;
	
TrapStatementRule
	: KWTRAP AttributeSpecificationOpt StatementBlockRule
	;
	
AttributeSpecificationOpt
	: /* empty */
	| ATTRIBUTESPECIFICATION
	;
	
FinallyStatementRule
	: KWFINALLY StatementBlockRule
	;
	
FlowControlStatementRule
	: FlowControlKeyword FlowControlParameterOpt
	| KWRETURN PipelineRule
	;
	
FlowControlKeyword
	: KWBREAK
	| KWCONTINUE
	;
	
FlowControlParameterOpt
	: /* empty */
	| PROPERTYNAME
	| PropertyOrArrayReferenceRule
	;
	
FunctionDeclarationRule
	: FUNCTIONDECLARATION PARAMETERARGUMENT ParameterDeclarationExpressionRuleOpt CmdletBodyRule
	;
	
ParameterDeclarationExpressionRuleOpt
	: /* empty */
	| '(' ParameterDeclarationExpressionRule ')' { Match(@1,@3); }
	;
	
CmdletBodyRule
	: '{' ParameterDeclarationRuleOpt CmdletBodyProcessRulesOpt StatementListRule '}' { Match(@1,@5); }
	;
	
ParameterDeclarationRuleOpt
	: /* empty */
	| '(' ParameterDeclarationRule ')'
	;
	
CmdletBodyProcessRulesOpt
	: /* empty */
	| CmdletBodyProcessRules
	;
	
CmdletBodyProcessRules
	: CmdletBodyProcessRule
	| CmdletBodyProcessRules CmdletBodyProcessRule
	;
	
CmdletBodyProcessRule
	: KWBEGIN StatementBlockRule
	| KWPROCESS StatementBlockRule
	| KWEND StatementBlockRule
	;

ParameterDeclarationRule
	: PARAMETERDECLARATION '(' ParameterDeclarationExpressionRule ')' { Match(@2,@4); }
	;
	
ParameterDeclarationExpressionRule
	: ParameterWithInitializer
	| ParameterDeclarationExpressionRule COMMA ParameterWithInitializer
	;
	
ParameterWithInitializer
	: SimpleLValue InitializerOpt
	;
	
InitializerOpt
	: /* empty */
	| '=' ExpressionRule
	;
	
ExpressionRule
	: LogicalExpressionRule
	;
	
LogicalExpressionRule
	: BitwiseExpressionRule 
	| LogicalExpressionRule LogicalOperator BitwiseExpressionRule
	;
	
BitwiseExpressionRule
	: ComparisonExpressionRule
	| BitwiseExpressionRule BitwiseOperator ComparisonExpressionRule
	;
	
BitwiseOperator
	: BAND
	| BOR
	;
	
LogicalOperator
	: AND
	| OR
	;
	
ComparisonExpressionRule
	: AddExpressionRule
	| ComparisonExpressionRule ComparisonOperator AddExpressionRule
	;
	
AddExpressionRule
	: MultiplyExpressionRule
	| AddExpressionRule AdditionOperator MultiplyExpressionRule
	;
	
MultiplyExpressionRule
	: FormatExpressionRule 
	| MultiplyExpressionRule MultiplyOperator FormatExpressionRule
	;
	
MultiplyOperator
	: '*'
	| '/'
	| '%'
	;
	
AdditionOperator
	: '+'
	| '-'
	;
	
ComparisonOperator
	: EQ
	| NE
	| GE
	| GT
	| LT
	| LE
	| IEQ
	| INE
	| IGE
	| IGT
	| ILT
	| ILE
	| CEQ
	| CNE
	| CGE
	| CGT
	| CLT
	| CLE
	| LIKE
	| NOTLIKE
	| MATCH
	| NOTMATCH
	| ILIKE
	| INOTLIKE
	| IMATCH
	| INOTMATCH
	| CLIKE
	| CNOTLIKE
	| CMATCH
	| CNOTMATCH
	| CONTAINS
	| NOTCONTAINS
	| ICONTAINS
	| INOTCONTAINS
	| CCONTAINS
	| CNOTCONTAINS
	| ISNOT
	| IS
	| AS
	| REPLACE
	| IREPLACE
	| CREPLACE
	;

AssignmentOperator
	: '='			{ $$=(int)BinaryExpressionType.SimpleAssign; }
	| SELFPLUS		{ $$=(int)BinaryExpressionType.PlusAssign; }
	| SELFMINUS		{ $$=(int)BinaryExpressionType.MinusAssign; }
	| SELFMULT		{ $$=(int)BinaryExpressionType.MultiplyAssign; }
	| SELFDIV		{ $$=(int)BinaryExpressionType.DivisionAssign; }
	| SELFMOD		{ $$=(int)BinaryExpressionType.ModuloAssign; }
	;
		
RedirectionOperator
	: ERRORTOSTDOUT
	| STDOUTTOAPPENDFILE
	| STDOUTTOFILE
	| APPENDFILETOSTDIN
	| FILETOSTDIN
	| ERRORTOFILE
	| REDIRECTIONOPERATOR
	| ERRORTOAPPENDFILE
	| STDOUTTOAPPENDFILE
	;
	
FormatExpressionRule
	: RangeExpressionRule
	| FormatExpressionRule FORMATOPERATOR RangeExpressionRule
	;
	
RangeExpressionRule
	: ArrayLiteralRule
	| RangeExpressionRule RANGEOPERATOR ArrayLiteralRule
	;
	
ArrayLiteralRule
	: PostfixOperatorRule 
	| ArrayLiteralRule COMMA PostfixOperatorRule
	;
	
PostfixOperatorRule
	: LValueExpression PrePostfixOperator
	| PropertyOrArrayReferenceRule 
	;
	
PropertyOrArrayReferenceRule
	: LValueExpression
	| ValueRule
	;
	
PropertyOrArrayReferenceOperator
	: '[' ExpressionRule ']'									{ Match(@1,@3); }
	| '.' ReferenceRule
	;
	
ReferenceRule
	: PROPERTYNAME ParseCallRuleOpt
	| ValueRule
	;
	
ParseCallRuleOpt
	: /* empty */
	| ParseCallRule
	;
	
ParseCallRule
	: '(' ArrayLiteralRule ')'									{ Match(@1,@3); }
	;
	
ValueRule
	: '(' AssignmentStatementRule ')'							{ Match(@1,@3); }
	| '$' '(' StatementListRule ')'								{ Match(@1,@2,@4); }
	| ARRAYOPERATOR StatementListRule ')'						{ Match(@1,@3); }
	| CmdletBodyRule
	| HASHOPERATOR HashLiteralRule '}'							{ Match(@1,@3); }
	| UNARYOPERATOR PropertyOrArrayReferenceRule
	| PrePostfixOperator LValue
	| NUMBER
	| LITERALSTRING
	| ExpandableString
	| LValue
	;
	
ExpandableString
	: EXPANDABLESTRING ExpandableStringContent EXPANDABLESTRING	{ Match(@1,@3); }
	;
	
ExpandableStringContent
	: /* empty */
	| EXPANDABLESTRINGCONTENT
	| EXPANDABLESTRINGVARIABLE
	| ExpandableStringContent EXPANDABLESTRINGCONTENT
	| ExpandableStringContent EXPANDABLESTRINGVARIABLE
	;
	
HashLiteralRule
	: KeyExpression '=' PipelineRule
	| HashLiteralRule StatementSeparator KeyExpression '=' PipelineRule 
	;
	
KeyExpression
	: ValueRule
	;
	
PrePostfixOperator
	: PLUSPLUS
	| MINUSMINUS
	;
%%
