﻿using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.VisualStudio.Shell;
using System.Management.Automation.Host;
using Evaluant.PS4VS.Renderer;

namespace Evaluant.PS4VS
{
    /// <summary>
    /// This class implements the tool window exposed by this package and hosts a user control.
    ///
    /// In Visual Studio tool windows are composed of a frame (implemented by the shell) and a pane, 
    /// usually implemented by the package implementer.
    ///
    /// This class derives from the ToolWindowPane class provided from the MPF in order to use its 
    /// implementation of the IVsWindowPane interface.
    /// </summary>
    [Guid("6bc0c1c2-8bcd-4c07-a899-afb9d3932a8f")]
    public class MyToolWindow : ToolWindowPane
    {
        // This is the user control hosted by the tool window; it is exposed to the base class 
        // using the Window property. Note that, even if this class implements IDispose, we are
        // not calling Dispose on this object. This is because ToolWindowPane calls Dispose on 
        // the object returned by the Window property.
        private MyControl control;

        PSHostUserInterface userInterface;
        PSHostRawUserInterface rawUserInterface;

        public ConsoleControl InnerControl { get { return control.Console; } }

        /// <summary>
        /// Standard constructor for the tool window.
        /// </summary>
        public MyToolWindow() :
            base(null)
        {
            userInterface = Language.Resolver.UI;

            // Set the window title reading it from the resources.
            this.Caption = Resources.ToolWindowTitle;
            // Set the image that will appear on the tab of the window frame
            // when docked with an other window
            // The resource ID correspond to the one defined in the resx file
            // while the Index is the offset in the bitmap strip. Each image in
            // the strip being 16x16.
            this.BitmapResourceID = 301;
            this.BitmapIndex = 1;


            control = new MyControl();
        }

        /// <summary>
        /// This property returns the handle to the user control that should
        /// be hosted in the Tool Window.
        /// </summary>
        override public IWin32Window Window
        {
            get
            {
                return (IWin32Window)control;
            }
        }


        internal void Write(ConsoleColor foregroundColor, ConsoleColor backgroundColor, string value)
        {
            InnerControl.Write(value);
            //control.Write(foregroundColor.ToDrawingColor(), backgroundColor.ToDrawingColor(), value);
        }

        internal void WriteLine(ConsoleColor foregroundColor, ConsoleColor backgroundColor, string message)
        {
            InnerControl.WriteLine(message);
            //control.WriteLine(foregroundColor.ToDrawingColor(), backgroundColor.ToDrawingColor(), message);
        }

        internal string ReadLine()
        {
            return control.ReadLine();
        }
    }
}
